/*
 * n01
 *
 * game_history.c
 *
 * Copyright (C) 1996-2004 by Nakashima Tomoaki. All rights reserved.
 *		http://www.nakka.com/
 *		nakka@nakka.com
 */

/* Include Files */
#define _INC_OLE
#include <windows.h>
#undef  _INC_OLE
#include <tchar.h>

#include "..\n01_plugin.h"
#include "Memory.h"
#include "font.h"
#include "Message.h"

#include "resource.h"

/* Define */
#define TYPE_HEADER				0
#define TYPE_LEG				1

/* Global Variables */
extern HINSTANCE hInst;
extern TCHAR csv_path[];
extern TCHAR xls_path[];

typedef struct _VIEW_INFO {
	int type;

	SCORE_INFO *si;
	int leg;
} VIEW_INFO;

/* Local Function Prototypes */
static void get_control_pos(const HWND hDlg, RECT *c_rect);
static BOOL add_score_info(const HWND hlist, SCORE_INFO *si);
static BOOL add_score_list(const HWND hlist, SCORE_HISTORY *sh);
static void drow_list_item(const DRAWITEMSTRUCT *di, const VIEW_INFO *sel_vi);
static void set_enabled(const HWND hDlg, const SCORE_HISTORY *sh);
static BOOL CALLBACK option_game_history_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);

/*
 * get_control_pos - Rg[̃NCAgW擾
 */
static void get_control_pos(const HWND hDlg, RECT *c_rect)
{
	RECT rect;
	POINT pt;

	GetClientRect(hDlg, &rect);

	c_rect->right = c_rect->right - c_rect->left;
	c_rect->bottom = c_rect->bottom - c_rect->top;
	pt.x = c_rect->left;
	pt.y = c_rect->top;
	ScreenToClient(hDlg, &pt);
	c_rect->left = rect.right - pt.x;
	c_rect->top = rect.bottom - pt.y;
}

/*
 * add_score_info - XgLegǉ
 */
static BOOL add_score_info(const HWND hlist, SCORE_INFO *si)
{
	VIEW_INFO *vi;
	TCHAR buf[BUF_SIZE];
	int i, j;

	for (i = 0; i < si->current_leg; i++) {
		wsprintf(buf, message_get_res(IDS_STRING_OP_LIST_LEG), i + 1);
		j = SendMessage(hlist, LB_ADDSTRING, 0, (LPARAM)buf);
		vi = (VIEW_INFO *)mem_calloc(sizeof(VIEW_INFO));
		vi->type = TYPE_LEG;
		vi->si = si;
		vi->leg = i + 1;
		SendMessage(hlist, LB_SETITEMDATA, j, (LPARAM)vi);
	}
	return TRUE;
}

/*
 * add_score_list - XgɃACeǉ
 */
static BOOL add_score_list(const HWND hlist, SCORE_HISTORY *sh)
{
	VIEW_INFO *vi;
	TCHAR buf[BUF_SIZE];
	int i, j;

	for (i = 0; i < sh->list_count; i++) {
		_itot(sh->list[i].start_score, buf, 10);
		j = SendMessage(hlist, LB_ADDSTRING, 0, (LPARAM)buf);
		vi = (VIEW_INFO *)mem_calloc(sizeof(VIEW_INFO));
		vi->type = TYPE_HEADER;
		vi->si = &sh->list[i];
		vi->leg = 0;
		SendMessage(hlist, LB_SETITEMDATA, j, (LPARAM)vi);
		add_score_info(hlist, &sh->list[i]);
	}
	if (sh->si->history == FALSE && sh->si->current_leg > 0) {
		_itot(sh->si->start_score, buf, 10);
		j = SendMessage(hlist, LB_ADDSTRING, 0, (LPARAM)buf);
		vi = (VIEW_INFO *)mem_calloc(sizeof(VIEW_INFO));
		vi->type = TYPE_HEADER;
		vi->si = sh->si;
		vi->leg = 0;
		SendMessage(hlist, LB_SETITEMDATA, j, (LPARAM)vi);
		add_score_info(hlist, sh->si);
	}
	return TRUE;
}

/*
 * drow_list_item - XgACe̕`
 */
static void drow_list_item(const DRAWITEMSTRUCT *di, const VIEW_INFO *sel_vi)
{
	VIEW_INFO *vi;
	HANDLE hBrush;
	HFONT hfont, ret_font;
	HPEN pen, ret_pen;
	RECT rect;
	RECT header_rect, info_rect, p_rect[2];
	SIZE sz;
	COLORREF text_color;
	COLORREF back_color;
	TCHAR buf[BUF_SIZE];
	int i;

	if((int)di->itemData <= 0){
		hBrush = CreateSolidBrush(GetSysColor(COLOR_WINDOW));
		FillRect(di->hDC, &di->rcItem, hBrush);
		DeleteObject(hBrush);
		return;
	}

	vi = (VIEW_INFO *)di->itemData;

	rect = di->rcItem;
	i = (rect.right * 100) / 6;
	SetRect(&header_rect, rect.left + 4, rect.top, i * 1 / 100, rect.bottom);
	SetRect(&info_rect, i * 1 / 100, rect.top, i * 2 / 100, rect.bottom);
	SetRect(&p_rect[0], i * 2 / 100, rect.top, i * 4 / 100, rect.bottom);
	SetRect(&p_rect[1], i * 4 / 100, rect.top, rect.right, rect.bottom);

	if (vi->type == TYPE_HEADER && sel_vi != NULL && sel_vi->si == vi->si) {
		text_color = GetSysColor(COLOR_HIGHLIGHTTEXT);
		back_color = GetSysColor(COLOR_HIGHLIGHT);
	} else if (vi->type != TYPE_HEADER && sel_vi != NULL && sel_vi->si == vi->si) {
		text_color = GetSysColor(COLOR_WINDOWTEXT);
		back_color = RGB(255, 255, 200);
	} else {
		text_color = GetSysColor(COLOR_WINDOWTEXT);
		back_color = (vi->type == TYPE_HEADER) ? GetSysColor(COLOR_3DFACE) : GetSysColor(COLOR_WINDOW);
	}
	hBrush = CreateSolidBrush(back_color);
	FillRect(di->hDC, &rect, hBrush);
	DeleteObject(hBrush);

	SetTextColor(di->hDC, text_color);
	SetBkColor(di->hDC, back_color);

	switch (vi->type) {
	case TYPE_HEADER:
		hfont = font_create_menu(12, FW_BOLD, FALSE);
		ret_font = SelectObject(di->hDC, hfont);
		_itot(vi->si->start_score, buf, 10);
		DrawText(di->hDC, buf, lstrlen(buf), &header_rect, DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
		if (vi->si->leg_limit == 1) {
			wsprintf(buf, message_get_res(IDS_STRING_OP_LIST_SET), vi->si->current_set + 1);
			DrawText(di->hDC, buf, lstrlen(buf), &info_rect, DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
		}
		SelectObject(di->hDC, ret_font);
		DeleteObject(hfont);

		for (i = 0; i < 2; i++) {
			if (vi->si->player[i].com == TRUE) {
				wsprintf(buf, message_get_res(IDS_STRING_COM), vi->si->player[i].level + 1);
			} else {
				lstrcpy(buf, vi->si->player[i].name);
			}
			if (vi->si->player[i].legs > vi->si->player[!i].legs) {
				hfont = font_create_menu(12, FW_BOLD, TRUE);
				ret_font = SelectObject(di->hDC, hfont);
			} else {
				hfont = font_create_menu(12, FW_BOLD, FALSE);
				ret_font = SelectObject(di->hDC, hfont);
			}
			DrawText(di->hDC, buf, lstrlen(buf), &p_rect[i], DT_CENTER | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
			SelectObject(di->hDC, ret_font);
			DeleteObject(hfont);
		}
		break;

	case TYPE_LEG:
		hfont = font_create_menu(12, 0, FALSE);
		ret_font = SelectObject(di->hDC, hfont);
		wsprintf(buf, message_get_res(IDS_STRING_OP_LIST_LEG), vi->leg);
		DrawText(di->hDC, buf, lstrlen(buf), &info_rect, DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);

		GetTextExtentPoint32(di->hDC, TEXT("8888"), lstrlen(TEXT("8888")), &sz);
		for (i = 0; i < 2; i++) {
			p_rect[i].left += ((p_rect[i].right - p_rect[i].left) - sz.cx) / 2;
			p_rect[i].right = p_rect[i].left + sz.cx;
			p_rect[i].top++;
			p_rect[i].bottom--;
			if (vi->si->leg[vi->leg - 1].winner == i && vi->si->leg[vi->leg - 1].darts > 0) {
				SetTextColor(di->hDC, text_color);
				hBrush = CreateSolidBrush(text_color);
				_itot(vi->si->leg[vi->leg - 1].darts, buf, 10);
			} else {
				SetTextColor(di->hDC, GetSysColor(COLOR_3DSHADOW));
				hBrush = CreateSolidBrush(GetSysColor(COLOR_3DSHADOW));
				lstrcpy(buf, TEXT("-"));
			}
			DrawText(di->hDC, buf, lstrlen(buf), &p_rect[i], DT_CENTER | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
			FrameRect(di->hDC, &p_rect[i], hBrush);
			DeleteObject(hBrush);
		}
		SelectObject(di->hDC, ret_font);
		DeleteObject(hfont);

		pen = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_3DSHADOW));
		ret_pen = SelectObject(di->hDC, pen);
		MoveToEx(di->hDC, p_rect[0].right, rect.top + (rect.bottom - rect.top) / 2, NULL);
		LineTo(di->hDC, p_rect[1].left, rect.top + (rect.bottom - rect.top) / 2);
		SelectObject(di->hDC, ret_pen);
		DeleteObject(pen);
		break;
	}
}

/*
 * set_enabled - {^̏Ԑݒ
 */
static void set_enabled(const HWND hDlg, const SCORE_HISTORY *sh)
{
	if (SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETCURSEL, 0, 0) == -1) {
		EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_OUT), FALSE);
	} else {
		EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_OUT), TRUE);
	}
}

/*
 * option_game_history_proc - EBhEvV[W
 */
static BOOL CALLBACK option_game_history_proc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	SCORE_HISTORY *sh;
	VIEW_INFO *vi;
	RECT parent_rect, rect;
	POINT pt;
	HANDLE hFile;
	DWORD ret;
	TCHAR *p;
	int i;
	int left, top;
	static RECT list_rect, out_rect, cancel_rect;

	switch (msg) {
	case WM_INITDIALOG:
		if ((sh = (SCORE_HISTORY *)lParam) == NULL) {
			EndDialog(hDlg, FALSE);
			break;
		}
		SetWindowLong(hDlg, GWL_USERDATA, lParam);

		// Rg[̏W擾
		GetClientRect(hDlg, &rect);
		GetWindowRect(GetDlgItem(hDlg, IDC_LIST_SCORE_LIST), &list_rect);
		pt.x = 0;
		pt.y = list_rect.bottom;
		ScreenToClient(hDlg, &pt);
		list_rect.bottom = rect.bottom - pt.y;
		pt.x = 0;
		pt.y = list_rect.top;
		ScreenToClient(hDlg, &pt);
		list_rect.top = pt.y;

		GetWindowRect(GetDlgItem(hDlg, IDC_BUTTON_OUT), &out_rect);
		get_control_pos(hDlg, &out_rect);
		GetWindowRect(GetDlgItem(hDlg, IDCANCEL), &cancel_rect);
		get_control_pos(hDlg, &cancel_rect);

		// Z^O
		GetWindowRect(GetParent(hDlg), &parent_rect);
		GetWindowRect(hDlg, &rect);
		left = parent_rect.left + ((parent_rect.right - parent_rect.left) - (rect.right - rect.left)) / 2;
		if (left < 0) left = 0;
		top = parent_rect.top + ((parent_rect.bottom - parent_rect.top) - (rect.bottom - rect.top)) / 2;
		if (top < 0) top = 0;
		SetWindowPos(hDlg, 0, left, top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);

		SendMessage(hDlg, WM_SIZE, 0, 0);

		add_score_list(GetDlgItem(hDlg, IDC_LIST_SCORE_LIST), sh);
		SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_SETCURSEL, 0, 0);
		set_enabled(hDlg, sh);
		break;

	case WM_CLOSE:
		for (i = 0; i < SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETCOUNT, 0, 0); i++) {
			vi = (VIEW_INFO *)SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETITEMDATA, i, 0);
			if (vi != NULL) {
				mem_free(&vi);
				SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_SETITEMDATA, i, 0);
			}
		}
		EndDialog(hDlg, FALSE);
		break;

	case WM_SIZE:
		GetClientRect(hDlg, &rect);

		MoveWindow(GetDlgItem(hDlg, IDC_LIST_SCORE_LIST), 0, list_rect.top, rect.right, (rect.bottom - list_rect.bottom) - list_rect.top, TRUE);
		MoveWindow(GetDlgItem(hDlg, IDC_BUTTON_OUT), rect.right - out_rect.left, rect.bottom - out_rect.top, out_rect.right, out_rect.bottom, TRUE);
		MoveWindow(GetDlgItem(hDlg, IDCANCEL), rect.right - cancel_rect.left, rect.bottom - cancel_rect.top, cancel_rect.right, cancel_rect.bottom, TRUE);

		InvalidateRect(hDlg, NULL, FALSE);
		UpdateWindow(hDlg);
		break;

	case WM_MEASUREITEM:
		if((UINT)wParam == IDC_LIST_SCORE_LIST){
			((LPMEASUREITEMSTRUCT)lParam)->itemHeight = 24;
		}
		return TRUE;

	case WM_DRAWITEM:
		if((UINT)wParam == IDC_LIST_SCORE_LIST){
			vi = NULL;
			if (SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETCURSEL, 0, 0) != -1) {
				vi = (VIEW_INFO *)SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETITEMDATA, SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETCURSEL, 0, 0), 0);
			}
			drow_list_item((LPDRAWITEMSTRUCT)lParam, vi);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_LIST_SCORE_LIST:
			if (HIWORD(wParam) == LBN_DBLCLK) {
				SendMessage(hDlg, WM_COMMAND, IDC_BUTTON_OUT, 0);
			} else if (HIWORD(wParam) == LBN_SELCHANGE) {
				InvalidateRect(GetDlgItem(hDlg, IDC_LIST_SCORE_LIST), NULL, FALSE);
				UpdateWindow(GetDlgItem(hDlg, IDC_LIST_SCORE_LIST));
			}
			break;

		case IDC_BUTTON_OUT:
			if (SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETCURSEL, 0, 0) == -1) {
				break;
			}
			vi = (VIEW_INFO *)SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETITEMDATA, SendDlgItemMessage(hDlg, IDC_LIST_SCORE_LIST, LB_GETCURSEL, 0, 0), 0);
			if (vi == NULL) {
				break;
			}
			// CSV擾
			p = (TCHAR *)SendMessage(GetParent(hDlg), WM_N01_GET_CSV, 0, (LPARAM)vi->si);
			if (p == NULL) {
				break;
			}
			//CSVۑ
			hFile = CreateFile(csv_path, GENERIC_READ | GENERIC_WRITE, 0, 0, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
			if(hFile == NULL || hFile == (HANDLE)-1){
				SendMessage(GetParent(hDlg), WM_N01_FREE_CSV, 0, (LPARAM)p);
				break;
			}
			WriteFile(hFile, p, sizeof(TCHAR) * lstrlen(p), &ret, NULL);
			CloseHandle(hFile);
			// CSV
			SendMessage(GetParent(hDlg), WM_N01_FREE_CSV, 0, (LPARAM)p);
			// J
			if ((i = (int)ShellExecute(hDlg, (LPCTSTR)NULL, (LPCTSTR)xls_path, NULL, NULL, SW_SHOWNORMAL)) <= 32) {
				TCHAR err_str[BUF_SIZE];

				lstrcpy(err_str, message_get_res(IDS_STRING_ERR_EXCEL));
				message_get_error(GetLastError(), err_str + lstrlen(err_str));
				MessageBox(hDlg, err_str, TEXT("Error"), MB_ICONERROR);
			}
			break;

		case IDCANCEL:
			SendMessage(hDlg, WM_CLOSE, 0, 0);
			break;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

/*
 * show_game_history - Q[
 */
BOOL show_game_history(const HINSTANCE hInst, const HWND hWnd, SCORE_HISTORY *sh)
{
	return DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_DIALOG_GAME_HISTORY), hWnd, option_game_history_proc, (LPARAM)sh);
}
/* End of source */
